/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
**
** NAME
**
**     gxdialog, gxinput
**
** PURPOSE
**
**     Extensions to gtk: convenience dialogs
**
** CALLING SEQUENCE (gxdialog):
**
**     #include "gxdialog.h"
**     result = gxdialog ("type", "message_string", "win_title");
**
** INPUT
**
**     type can be one of the following strings: 
**          "error"
**          "info"
**          "question"
**          "warning"
**          "about"
**          "help"
**          "gnome"
**          "kde"
**
**     message_string is the text to display in the dialog box, and may 
**          contain newline (\n) characters for multi-line messages.
**
**     win_title is the window title.  Set this to NULL to use a default title.
**
** OUTPUT:
**
**     result (gint) is the return value.  All dialogs except question have
**          only a "Dismiss" button, and return the value 0.  A question dialog
**          has "Ok" and "Cancel" buttons, and returns 1 for "Ok", and 0
**          for "Cancel".
**         
**
** CALLING SEQUENCE (gxinput):
**
**     #include "gxdialog.h"
**     result = gxinput ("message_string", "initial_text", "win_title");
**
** INPUT:
**
**     message_string is the text to display in the dialog box, and may 
**          contain newline (\n) characters for multi-line messages.
**
**     initial_text is the default text to display in the input area.  Set
**          this to NULL if you do not want to use any default text.
**
**     win_title is the window title.  Set this to NULL to use a default title.
**
** OUTPUT:
**
**     result (gchar *) is a pointer to the text that was entered into
**          the dialog, or NULL if the dialog was canceled, or no text
**          was entered.  
**
** MODIFICATION HISTORY
**
**     v0.9: Written, mallozzir@cspar.uah.edu, 1998 August
**
** * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
**
** This program is free software; you can redistribute it and/or
** modify it under the terms of the GNU Library General Public
** License as published by the Free Software Foundation; either
** version 2 of the License, or (at your option) any later version.
**
** This software is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
** Library General Public License for more details.
**
** You should have received a copy of the GNU Library General Public
** License along with this library; if not, write to the Free
** Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
**
**  This software is in no way affiliated with the University of Alabama,
**  or NASA and the US government.
**
** * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

#include <gtk/gtk.h>
#include "gxdialog.h"


/*
** Include the pixmaps explicitly to make it easier for others to compile 
** gxdialog into other applications.
*/


/* 
** #include "error.xpm"
*/
static gchar * error_xpm[] = {
/* width height ncolors chars_per_pixel */
"50 50 5 1",
/* colors */
" 	s background	c None",
".	c #F81040",
"X	c #B02020",
"o	c black",
"O	c #686868",
/* pixels */
"                                                  ",
"                                                  ",
"                                                  ",
"                                                  ",
"                                                  ",
"                                                  ",
"                                                  ",
"                                                  ",
"                                                  ",
"                                                  ",
"                      .......                     ",
"                   ...XXXXXXX...                  ",
"                  ..XXXXXXXXXXXX.                 ",
"                ...XXXXXXXXXXXXXX..               ",
"               ..XXXXXXoooooXXXXXXXXX             ",
"              ..XXXXoooOOOOOoooXXXXXXo            ",
"              .XXXXX.OOOOOOOOOOoXXXXXo            ",
"             ..XXXXX..OO    OOOO.XXXXXo           ",
"            ..XXXXXXX..         O.XXXXXo          ",
"            .XXXo..XXX..          .XXXXoO         ",
"            .XXXoO..XXX..         .XXXXoO         ",
"           .XXXXoOO..XXX..        .XXXXXoO        ",
"           .XXXoOO  ..XXX..        .XXXXoOO       ",
"           .XXXoOO   ..XXX..       .XXXXoOO       ",
"           .XXXoOO    ..XXX..      .XXXXoOO       ",
"           .XXXoOO     ..XXX..     .XXXXoOO       ",
"           .XXXoOO      ..XXX..    .XXXXoOO       ",
"           .XXXXoOO      ..XXX..  .XXXXXoOO       ",
"            .XXXoOO       ..XXX.. .XXXXoOOO       ",
"            .XXXoOO        ..XXX...XXXXoOOO       ",
"            .XXXXoO         ..XXXXXXXXXoOO        ",
"             .XXXX.O         ..XXXXXXXoOOO        ",
"              .XXXX.          .XXXXXXoOOOO        ",
"              .XXXXX...     ...XXXXXXoOOO         ",
"               XXXXXXXX.....XXXXXXXXoOOOO         ",
"               XXXXXXXXXXXXXXXXXXXXoOOOO          ",
"                ooXXXXXXXXXXXXXXXooOOOOO          ",
"                  oXXXXXXXXXXXXXoOOOOOO           ",
"                   oooXXXXXXXoooOOOOOO            ",
"                    OOoooooooOOOOOOOO             ",
"                      OOOOOOOOOOOO                ",
"                        OOOOOOO                   ",
"                                                  ",
"                                                  ",
"                                                  ",
"                                                  ",
"                                                  ",
"                                                  ",
"                                                  ",
"                                                  "};

/*
** #include "info.xpm"
*/
static gchar * info_xpm[] = {
/* width height ncolors chars_per_pixel */
"50 50 3 1",
/* colors */
" 	s background	c None",
".	c #5050F8",
"X	c #686868",
/* pixels */
"                                                  ",
"                                                  ",
"                                                  ",
"                                                  ",
"                                                  ",
"                                                  ",
"                                                  ",
"                                                  ",
"                                                  ",
"                                                  ",
"                       ...                        ",
"                      .....                       ",
"                     .......                      ",
"                     .......X                     ",
"                     .......XX                    ",
"                      .....XXX                    ",
"                       ...XXXX                    ",
"                        XXXXX                     ",
"                         XXX                      ",
"                       .....                      ",
"                   .........                      ",
"                   .........XX                    ",
"                     X......XX                    ",
"                     X......XX                    ",
"                      ......XX                    ",
"                      ......XX                    ",
"                      ......XX                    ",
"                      ......XX                    ",
"                      ......XX                    ",
"                      ......XX                    ",
"                      ......XX                    ",
"                      ......XX                    ",
"                      ......XX                    ",
"                      ......XX                    ",
"                      ......XX                    ",
"                      ......XX                    ",
"                      ......XX                    ",
"                   ...........                    ",
"                   ...........                    ",
"                     XXXXXXXXXXX                  ",
"                     XXXXXXXXXXX                  ",
"                                                  ",
"                                                  ",
"                                                  ",
"                                                  ",
"                                                  ",
"                                                  ",
"                                                  ",
"                                                  ",
"                                                  "};

/* 
** #include "warning.xpm"
*/
static gchar * warning_xpm[] = {
/* width height ncolors chars_per_pixel */
"50 50 4 1",
/* colors */
" 	s background	c None",
".	c black",
"X	c #F8FC00",
"o	c #686868",
/* pixels */
"                                                  ",
"                                                  ",
"                                                  ",
"                                                  ",
"                                                  ",
"                                                  ",
"                                                  ",
"                                                  ",
"                                                  ",
"                                                  ",
"                       ...                        ",
"                      .XXX.                       ",
"                     .XXXX.oo                     ",
"                     .XXXXX.oo                    ",
"                    .XXXXXX.oo                    ",
"                    .XXXXXXX.oo                   ",
"                   .XXXXXXXX.oo                   ",
"                   .XXX...XXX.oo                  ",
"                  .XXX.....XX.oo                  ",
"                  .XXX.....XXX.oo                 ",
"                 .XXXX.....XXX.oo                 ",
"                 .XXXX.....XXXX.oo                ",
"                .XXXXX.....XXXX.oo                ",
"                .XXXXX.....XXXXX.oo               ",
"               .XXXXXX.....XXXXX.oo               ",
"               .XXXXXX.....XXXXXX.oo              ",
"              .XXXXXXXX...XXXXXXX.oo              ",
"              .XXXXXXXX...XXXXXXXX.oo             ",
"             .XXXXXXXXX...XXXXXXXX.oo             ",
"             .XXXXXXXXX...XXXXXXXXX.oo            ",
"            .XXXXXXXXXXXXXXXXXXXXXX.oo            ",
"            .XXXXXXXXXX...XXXXXXXXXX.oo           ",
"           .XXXXXXXXXX.....XXXXXXXXX.oo           ",
"           .XXXXXXXXXX.....XXXXXXXXXX.oo          ",
"          .XXXXXXXXXXXX...XXXXXXXXXXX.oo          ",
"          .XXXXXXXXXXXXXXXXXXXXXXXXXXX.oo         ",
"          .XXXXXXXXXXXXXXXXXXXXXXXXXXX.oo         ",
"          .............................oo         ",
"             oooooooooooooooooooooooooooo         ",
"               ooooooooooooooooooooooooo          ",
"                                                  ",
"                                                  ",
"                                                  ",
"                                                  ",
"                                                  ",
"                                                  ",
"                                                  ",
"                                                  ",
"                                                  ",
"                                                  "};

/* 
** #include "question.xpm"
*/
static gchar * question_xpm[] = {
/* width height ncolors chars_per_pixel */
"50 50 3 1",
/* colors */
" 	s background	c None",
".	c #208820",
"X	c #686868",
/* pixels */
"                                                  ",
"                                                  ",
"                                                  ",
"                                                  ",
"                                                  ",
"                                                  ",
"                                                  ",
"                                                  ",
"                                                  ",
"                                                  ",
"                                                  ",
"                     ........                     ",
"                   ............                   ",
"                  ......XX......                  ",
"                 ......XXXX......                 ",
"                 ......XX  ......                 ",
"                 .....XX   ......X                ",
"                  ....XX   ......X                ",
"                  XXXXX   .......X                ",
"                   XXXX  .......XX                ",
"                        .......XXX                ",
"                       ......XXXX                 ",
"                      ......XXXX                  ",
"                     ......XXX                    ",
"                    ......XXX                     ",
"                    ......XX                      ",
"                    .......                       ",
"                    .........                     ",
"                     ........                     ",
"                     X......XX                    ",
"                      XXXXXXXX                    ",
"                       XXXXXX                     ",
"                       ...                        ",
"                      .....                       ",
"                     .......                      ",
"                     .......                      ",
"                     .......X                     ",
"                     ......XX                     ",
"                      ....XXX                     ",
"                      XXXXXX                      ",
"                       XXXX                       ",
"                                                  ",
"                                                  ",
"                                                  ",
"                                                  ",
"                                                  ",
"                                                  ",
"                                                  ",
"                                                  ",
"                                                  "};


/* 
** #include "gtk-block.xpm"
*/
/* XPM */
static char * gtk_block_xpm[] = {
"50 50 16 1",
" 	s None	c None",
".	c #A6ACAD",
"X	c #27505F",
"o	c #8E9193",
"O	c #192E3F",
"+	c #526AA6",
"@	c #2A0E12",
"#	c #0A0D16",
"$	c #3A71BA",
"%	c #5D1D20",
"&	c #A56369",
"*	c #E85352",
"=	c #3CA43F",
"-	c #42DF43",
";	c #F04D4E",
":	c #9E393C",
"                                                  ",
"                          .                       ",
"                      Xo.oXOOo                    ",
"                   .+O@O##OX###o                  ",
"                  +O@@###X++$X##Oo                ",
"                oO####OX+$$$$$+O##X               ",
"              +@####XX++$+$$$$$$+##@o             ",
"            oO###OO+OOOO###OOX+$$$X##O.           ",
"          .O###OX+O############OO+$+O##+.         ",
"        .X###OX++#####OX+XXO#####O$$$+O@#o        ",
"       +@##X+$$XO###OX+$$$$$+O####X$$$$+O#O.      ",
"   . o@##X+$$$+O###O+$$$$$$$+$X###O$$$$$+X#@X     ",
"   ++##X++$$$$$O###X$$$$$+X$$$$XO#X$$$$$$$+X#@o   ",
"   +@@%@+$$$$$$O###O$$$$$O##X+$+$$$$$$$$$$$++XO   ",
"   o@@&%#X$$$+$+O##O$$$$X####O+$$$$$$$$$$+XX$X&   ",
"   .@%**%@O+$$$$+#@#O+$$$O#####X$$$$$$$$XO$=-X+   ",
"   .#%*;*:@#O+$$$X###OX$++####O+$$$$$+OO$----Oo   ",
"   .#%*;;*:@#@X+$$O##########O$$$$$+OOX=-----Oo   ",
"   .#%*;;;;*@@@O++$$O######O$$$$+XO#X=------=#.   ",
"   o#%*;;;;**%%@@O$++$+XXX$$++XO#OX---------=#.   ",
"   .#%*;;;****%%:@@X+$$$$$+$O###$-----------$#    ",
"   o#%*;;*%:**;:%&:@OX$$$+OOOOX=------==X=--X@    ",
"   .#@;;*:@@%:**:%:*%@X++X##X=-------=OOX---OX    ",
"   o@@*;*%@##@%:*:%:*:@#XOX=---------O#O=--=#&    ",
"   o@%:;*@#####@%*:%%::#==--=$X=----O##=---=#+    ",
"   o@%:;;:@@#####@%%:%:#X---=##=---X##X----=#o    ",
"   o@%:;;;*:@@#####@:*:@O---X##X--X##O=----X#.    ",
"   o#%:;;;;;:#######:*:@#=--X##X=X##O=-----X#     ",
"   o#%:;;;;;:######@:*:OO=--O##XXO##$------O#     ",
"   o@%:;;;;;%###:%@@:*:#O--=O##X###X------=#@     ",
"   .#%:;;;;*%@#@:;;::*:##=--###O###O=-----=#+     ",
"   .#%:;;;;*%##@*;;;;;:#O=--O#######O$----$#+     ",
"   o@%:;;;;*%##@;;;;*;:#O--=####XXO###O$--X#o     ",
"   o#%:;;;;*%#@%*;;*;*:##---###=--=#####X-X#.     ",
"   .@@:;;;;*%##%*;;;;*:#O--=O##----==##O=-X#.&    ",
"   .@@;;;;;;%@#:*;;*;*:#O--=##O------===--X##O    ",
"    %%:**;;;:%#:*;;;**:#O--=##O----------=O#Xo    ",
"    .O#%:**;;;%:*;*;;;:#X--$##$---------=O#@+.    ",
"     o@@@%*;;;*;;;;;;;%#$--=#O=-------=X##O.      ",
"       +X@%:;;;;;;*;**%#$--=X=-------XO#@o        ",
"        o+@@:*;;;;;;;*%#=-==-------=O##+          ",
"          +O@%**;;*;**@#=--------=X##+.           ",
"           o%#%**;;*;*@#=-------X##X              ",
"            .+@@:*;;;&@#=-----$O#O.               ",
"              oO@:*;;:@#----=O#O.                 ",
"                +@%**:##---X#O&                   ",
"                 .X@:%#O$XO#+.                    ",
"                   X###@X@X.                      ",
"                    o@#@Xo                        ",
"                     .@O.                         "};


/* 
** #include "help.xpm"
*/
/* XPM */
static char * help_xpm[] = {
"50 50 5 1",
" 	s None	c None",
".	c #C0C0C0",
"X	c #808080",
"o	c white",
"O	c #F80000",
"                                                  ",
"                       ....                       ",
"                   ....X X...X..X...              ",
"                  ..       ..X.XX  ..             ",
"                 ..                 .X            ",
"                 ..                 .X            ",
"               ... .XXX...X....X    X.XX          ",
"              .XXXX................X .XX          ",
"             XXXX...................X.X           ",
"            OO......................OOOO          ",
"           OOOX....................OOOOOOO        ",
"      X.  OOOOO..................OOOOOOOOOO       ",
"      X.  OOOOOO.................OOOOOOOOOO       ",
"      ....OOOOOOO......XXXXXX...OOOOOOOOOOO       ",
"    X.  XXOOOOOOOOXXXXXXXXXXXXX.OOOOOOOOXXXX      ",
"  ....  XXXOOOOOOOOX....   ....XOOOOOOO...X..     ",
"  ...  X...XOOOOOOOOX          XOOOOOO.....X .    ",
"  ..   X....XOOOOOO             XOOOO......X. ..  ",
"  ..  X......XOOOO               OOO.......X. ... ",
"  ..  X......XOOOO               OOO.......X. ... ",
"  ... X......XXXX                  X........X   . ",
"  ... X......XX..                   ........X.  .X",
"  ... X......X.XX                   X.......X.   .",
" ...  .......X.                     X......X..   .",
" ...  ......XX.                     X.......X.   .",
" .    .......X.                     X.......X.   .",
" .    .......X.                     X.......X.   .",
" ...  ......XXX                     X.......X.  .X",
" X..  X.....XXX..                   X.......X.  . ",
"  ..  X......XX..                   ..........  . ",
"  ..X X.......XXXO                 X.........   . ",
"  XX. X........XXO               OOOOO.......   ..",
"    X. X.......OOOO             OOOOOOOX...X.   X ",
"    X. X.......OOOO             OOOOOOOX...X.   X ",
"     XXXXX....OOOOOOX          XOOOOOOOOXX..    X ",
"      XX....XOOOOOOO...XX.XXXX...OOOOOOOOOX. X..X ",
"        XXXOOOOOOOOX...............OOOOOOOO  ...  ",
"        OOOOOOOOOOO.................OOOOOOO.XX    ",
"          OOOOOOOO..................XOOOOO  X     ",
"           OOOOOOX...................OOOXX        ",
"           OOOOOOX...................OOOXX        ",
"            OOOXXX.................X.XOO          ",
"             X ..XXXX..........X.....X            ",
"            X.   ........XXXX...... X             ",
"            X..     ...........    X              ",
"              ...                XX               ",
"               XXX......       .X                 ",
"               XXX......       .X                 ",
"                  ...ooX......X                   ",
"                        .XXXXX                    "};


/* 
** #include "gnome.xpm"
*/
/* XPM */
static char *gnome_xpm[] = {
"50 50 17 1",
" 	s None	c None",
".	c #DBD7D4",
"X	c white",
"o	c #B2A79F",
"O	c #85796F",
"+	c #EAE8E6",
"@	c #94877C",
"#	c #3D3833",
"$	c #504841",
"%	c #A5978C",
"&	c #CEC8C3",
"*	c #73685F",
"=	c #C0B8B0",
"-	c #1B1917",
";	c black",
":	c #625950",
">	c #2C2723",
"                                                  ",
"                      .XX.oO                      ",
"                     .XX++@#$                     ",
"                    %X+XX&#*#*                    ",
"            X+&%O   .+=X.oO$-;O                   ",
"          @XX&=@:O  .X+..@O#-;*                   ",
"          @&++oO#$*@+.Xo%%@:-;;O                  ",
"          %.=&&O$#:O=%=o@O*$;;;O                  ",
"          @@@o@*>;;:O*OOOO$>;;;*                  ",
"          O@O**>;;;>*:$$$$>;;;>*                  ",
"          @*@$>;;;;#O*##>-;;;;-O                  ",
"      +XX@  :>;;;;>$  *>#>;;;-:                   ",
"     @XXXO@ @*$->>$O   O:$##$*@                   ",
"     =..&O#*   @O@     &+.&=%@                    ",
"     &=&**-;O       &+=XX.+&%%%OO                 ",
"     @o%@>;;>O    .XX&X&+.X@%o@:*#O               ",
"     O%O#-;;;O   &.X+++o++&&@+@*$>$               ",
"     XO:#;;;;*   XX.&..@O==%%=@:>;;O              ",
"     XX*:-;-$O  ..X.==%@*%oO%>:$-;;:              ",
"   X=+&@O***@ .XXX&=%oOO@=oO@O->;;;*              ",
"  XXXX&**O   @XX+Xo%@*oO=@o*o*$;;;;*              ",
"  &++.=%$#O  &++X.%o@o@*%o%oO:>;;;>O              ",
"  Oo&==*#>:@ &XX==%OoO%%oo@O:$;;;;$@              ",
"  @*::#>;;>*@+=X&@*%o@%o%O:$;;;;;#O               ",
"   *:>-;;;;:Oo.=%%@O:*O*O$>;;;;;#*                ",
"     *:;;;#**.+.%o%oo:%#-;;;;;>$*                 ",
"      @*$:O%O&+&=%%@@@#>;;;;>#*O                  ",
"            O=..%@o@%*#;;;>:@      .X&&+o=o@      ",
"            Ooo.=%@@%:-;;>:@     .+X.+.&.o@:*     ",
"            O&..oo@@@:;;;#O     +X+.&+&==o@#>*@   ",
"             =.&==%OO$;;>:      .oo...@o=%:>;#*   ",
"             %%%=oooO:;;#*      OO%@%@@o%*:;;>*   ",
"             @oooo=@O*>;;*       O:%o==@*@#;;>:   ",
"             @o=oo=@O*:>-*@       Oo=o%%O$>;;-*   ",
"              %@=%%@@O*$$::        *++oo@:;;;#O   ",
"              O:%o%%%%@O:@%@%=%.+.XXXoo@*#;;;:@   ",
"              @%**o%%O@@O%*@=&&+&+X+&&%%:-;;>O    ",
"               OO*Oo@%O@@oo@%&=&++X.=%%*#;;;$     ",
"               @****@@%@@%O@oOo==&*%%o::;;;#*     ",
"                O:::$@O@@%@%ooo=%:oo@$$-;;;:      ",
"                 *$::$@@@@o=%O@%o%%%@:-;;;>O      ",
"                 @*$$$:**O%OO%O@%%%@:>;;;>:       ",
"                   O$>>#$$:****O*O:$;;;;-:@       ",
"                    O:>>--##$$:$:$#;;;;;#O        ",
"                      O$-;;;;;;;;;;;;;-$@         ",
"                       @*:$;;;;;;;;;>>*O          ",
"                           O*$$####:*@            ",
"                              @O@@@               ",
"                                                  ",
"                                                  "};


/* 
** #include "kde.xpm"
*/
/* XPM */
static char *kde_xpm[] = {
"71 62 30 1",
" 	s None	c None",
".	c #BBBBBB",
"X	c #9D9D9D",
"o	c #898989",
"O	c #757575",
"+	c #616161",
"@	c #303030",
"#	c #090909",
"$	c #B0B0B0",
"%	c #BBA69D",
"&	c #E2E2E2",
"*	c #454545",
"=	c white",
"-	c #F5F5F5",
";	c #D7D7D7",
":	c #9D7545",
">	c #452700",
",	c #B07F45",
"<	c #F5E2A6",
"1	c #C49D61",
"2	c #BBB06C",
"3	c #F5FFCE",
"4	c #6C3A09",
"5	c #E2CEBB",
"6	c #CEBB9D",
"7	c #FFCE93",
"8	c #935813",
"9	c #E2BB89",
"0	c #BB7527",
"q	c #EBA661",
"                                                                       ",
"                             .XoOO+++Oo                                ",
"                            X+@#@@@@@@@+           O++Oo               ",
"                           $+@@++OO%$.&*o         O@o+@@*+Oo           ",
"                           o@==========*X        o@o===-X+@@*+o        ",
"                           o#==========@o       X**========&X+*o       ",
"                           o@==========@o      $+@&===========o+       ",
"                           o@==========@o     .+#$===========$@o       ",
"                O++X.      o#==========@O    ;O@O===========&@+$       ",
"               +@X+*O$     O@==========@+   ;O@#============@*%        ",
"              +@X=-O@+X   X*@==========@@o  o@#-===========+@o         ",
"             O@X====.@*OOO*@#==========#@@++#@.===========o@O          ",
"            %*X======-+@@@*@@==========@@@@#@+===========.@+           ",
"            o+=========X*+.X@==========*o$*@@===========&@+            ",
"            $*X=======-$$=&X@==========*OX+#&===========@*X            ",
"            .O@&=====-$&==-o@==========@*+@.===========+@o             ",
"             %*o====-$-===$@#==========@@@+===========%@+              ",
"              o@o==-.-==-o#@@==========#@#===========;#+               ",
"               O@&=.&==-+@+:>==========@@&==========-@@                ",
"               X**;.===+@,<1>==========#%===========*#o                ",
"                +@+-==o@2331@==========o===========O@+                 ",
"                +*$==;@+--32@=====================.@@O                 ",
"               o@+-=-+@-=--2@====================-#@#+                 ",
"               O*o==.@X===-1@====================X@@@+                 ",
"              o**;==O@====324=====================+#@#O                ",
"           Xo+*@*==-@O====-24==========-==========-@@@@*OOo            ",
"        o+*@@+OXO==.@;-==--24==========*===========.@#+++*@*+O         ",
"       O+O%&===&X==X@&3--3-24==========@O===========O@+$..$XO+o        ",
"       *;======;.==o*3-3=3324==========#@.===========@@OX$.$$O+        ",
"       *;======;.==O+3333-324==========>>>-==========&@*oX$$$O+        ",
"       *.======;;==O+<3333324==========>>>*===========o@+o$$$o+        ",
"       *.======.;==O+5<3<3<24==========>>4>6===========*@+XX$o*        ",
"       *.======;.==o+7<<<<<14==========>48>>&==========-@@OXXo*        ",
"       *.======&X==$*9<<<7<,4==========>408>+===========.@*oXo*        ",
"       O*+OX.-=-+==;*277777,>==========>4004>X===========+@@@*O        ",
"        o**@***+@&==+,99999,>==========>4008>>&===========#@@O$        ",
"           oo+*@*.==X+9999q,>==========>40008>4===========&@*o         ",
"               +@+==&+1qqqq0>==========>400084>o===========X@O         ",
"               o@@&==oO2qq10>==========>400008>@;===========**X        ",
"                +*o==&O21qq8>==========>400000:+@============@+$       ",
"                O@@&==.O11,0>==========>40000,;O@O===========.@o       ",
"                o@@+===%o118>==========>4000,&-X+@.===========OO       ",
"                O@;*$---$o18>==========>400,3=;$O@@-=======;o++o       ",
"               X*+-;*;---$X8>.====&.O*@>801--;..X@#+====.O*@*+X        ",
"              .+@&--$*;-&-$O**@@>>>>>>>4,5=-;...+@*@.O@#@*OX           ",
"              O@$&&&-X@;&-;$o*+844888:15&=&..;$O@oO*@@@@*X             ",
"              *+&&&&&-X@$;;;..X%..;;&&-&&..;.$+@X$XoOO+@X              ",
"             O@;&&&&&;&$@+;;;;;;;;;;;;;.;.;$X@@X$$$$XXXo@o             ",
"             +o;;;&&;&;&X@@o;;;;.;.;;.....$+@@o$$$$X$XXXOO             ",
"             O*;;;;;;;;o@#@@@+X;;;;...;.X+@@@@@o$$X$XXXX+O             ",
"              **;;;;;;+@*O+@@@@#@*+O+*@#@@*@++*@+X$XXXX+*X             ",
"              X**;;;X*@+    O@@XoO++++Oo@@+o   O@*oXXXO@o              ",
"               o@+;O@@O      o@o.......o@O.     O*@OXO@O               ",
"                o*+*+o       $+O.;.....o+$       X+@+*O                ",
"                  oo          O+......$O+.        $ooX                 ",
"                              o@..$..$$+O                              ",
"                              X@X..$$$$+O                              ",
"                              $*o$$$$$$*o                              ",
"                              .+O$$$$X$@o                              ",
"                              ;o+XXoXoo*$                              ",
"                              &$O*****+o.-                             ",
"                              -;.%XXX%%.&                              "};



#define GX_CANCEL 0
#define GX_OK     1

static void usage();

#ifdef GXDIALOG_DEMO
void demo (GtkWidget *widget, gpointer data);
#endif 


typedef struct {
    gboolean  get_another;
    gpointer  retval;
} STATE_S;

STATE_S  *state;


/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
static void dialogCB (GtkWidget *widget, gpointer data) 
{      
    state->retval = data;
    state->get_another = FALSE;
    
}


/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
static void inputCB (GtkWidget *widget, gpointer data) 
{      
    GtkWidget *entry;
    
    
    entry = gtk_object_get_user_data (GTK_OBJECT (widget));
    
    state->retval = "";
    if (GPOINTER_TO_INT (data) == GX_OK)    
       state->retval = (gpointer) gtk_entry_get_text (GTK_ENTRY (entry));	
       
    state->get_another = FALSE;
    
}


/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
gint gxdialog (const gchar *type_str, const gchar *message_str,
    const gchar *utitle)
{
    GtkWidget *window;
    GtkWidget *vBox, *hBox, *w;
    
    GtkWidget *pixmap;
    GdkPixmap *gdk_pixmap;
    GdkBitmap *gdk_mask;
    GtkStyle  *style;

    gchar **discriminator;
    gchar *title;
    gint  retval;


    /*
    ** Global state data.
    */ 
    state = g_malloc (sizeof (*state));

    /* 
    ** discriminator holds the array of char pointers obtained from the
    ** #include of the pixmaps.
    */
    discriminator = NULL;
    
    if (strcasecmp (type_str, "error") == 0) {
       discriminator = error_xpm;
       title = utitle ? (gchar *) utitle : "Error";
    }
    else
    if (strcasecmp (type_str, "info") == 0) {
       discriminator = info_xpm;
       title =  utitle ? (gchar *) utitle : "Infomation";
    }   
    else
    if (strcasecmp (type_str, "question") == 0) {
       discriminator = question_xpm;
       title =  utitle ? (gchar *) utitle : "Question";
    }   
    else        
    if (strcasecmp (type_str, "warning") == 0) {
       discriminator = warning_xpm;
       title =  utitle ? (gchar *) utitle : "Warning";
    }   
    else
    if (strcasecmp (type_str, "about") == 0) {
       discriminator = gtk_block_xpm;
       title =  utitle ? (gchar *) utitle : "About";
    }   
    else
    if (strcasecmp (type_str, "help") == 0) {
       discriminator = help_xpm;
       title =  utitle ? (gchar *) utitle : "Help";
    }   
    else
    if (strcasecmp (type_str, "gnome") == 0) {
       discriminator = gnome_xpm;
       title =  utitle ? (gchar *) utitle : "G . N . O . M . E";
    }   
    else
    if (strcasecmp (type_str, "kde") == 0) {
       discriminator = kde_xpm;
       title =  utitle ? (gchar *) utitle : "KDE";
    }   
    else {
       usage();
       return (0);
    }

    /*
    ** Event processing flag: process events from dialog while TRUE
    */       
    state->get_another = TRUE;

    /*
    ** Toplevel.
    */
    window = gtk_window_new (GTK_WINDOW_TOPLEVEL);    
    gtk_signal_connect (GTK_OBJECT (window), "destroy",
	GTK_SIGNAL_FUNC (gtk_widget_destroy), NULL);
    gtk_window_set_title (GTK_WINDOW (window), title);
    gtk_container_set_border_width (GTK_CONTAINER (window), 10);
    
    /* Adding the pixmap seems to make this call required ?? 
    ** Who knows, 'cause the documentation sure doesn't help :-(
    */
    gtk_widget_realize (window);

    vBox = gtk_vbox_new (FALSE, 10);
    gtk_container_add (GTK_CONTAINER (window), vBox);
    gtk_widget_show (vBox);    

    hBox = gtk_hbox_new (FALSE, 10);
    gtk_box_pack_start (GTK_BOX (vBox), hBox, FALSE, FALSE, 2);
    gtk_widget_show (hBox);

    style = gtk_widget_get_default_style ();
    gdk_pixmap = gdk_pixmap_create_from_xpm_d ( 
        window->window, &gdk_mask, &style->bg[GTK_STATE_NORMAL], discriminator);
    pixmap = gtk_pixmap_new (gdk_pixmap, gdk_mask);
    gtk_box_pack_start (GTK_BOX (hBox), pixmap, FALSE, FALSE, 2);
    gtk_widget_show (pixmap);
    
    w = gtk_label_new (message_str);
    gtk_box_pack_start (GTK_BOX (hBox), w, FALSE, FALSE, 2);
    gtk_label_set_justify (GTK_LABEL (w), GTK_JUSTIFY_LEFT);
    gtk_widget_show (w);

    /* - - - - - - - - - - */

    w = gtk_hseparator_new ();
    gtk_box_pack_start (GTK_BOX (vBox), w, FALSE, FALSE, 2);
    gtk_widget_show (w);

    hBox = gtk_hbox_new (TRUE, 10);
    gtk_box_pack_start (GTK_BOX (vBox), hBox, FALSE, FALSE, 2);
    gtk_widget_show (hBox);

    if (strcasecmp (type_str, "question") == 0) {

       w = gtk_button_new_with_label ("Ok");
       gtk_box_pack_start (GTK_BOX (hBox), w, TRUE, TRUE, 0);
       gtk_signal_connect (GTK_OBJECT (w), "clicked",
	   GTK_SIGNAL_FUNC (dialogCB), GINT_TO_POINTER (GX_OK));
       GTK_WIDGET_SET_FLAGS (w, GTK_CAN_DEFAULT);
       gtk_widget_grab_default (w);
       gtk_widget_show (w);
    
       w = gtk_button_new_with_label ("Cancel");
       gtk_box_pack_start (GTK_BOX (hBox), w, TRUE, TRUE, 0);
       gtk_signal_connect (GTK_OBJECT (w), "clicked",
	   GTK_SIGNAL_FUNC (dialogCB), GINT_TO_POINTER (GX_CANCEL));
       gtk_widget_show (w);

    }
    else {

       w = gtk_button_new_with_label ("Dismiss");
       gtk_box_pack_start (GTK_BOX (hBox), w, TRUE, TRUE, 0);
       gtk_signal_connect (GTK_OBJECT (w), "clicked",
           GTK_SIGNAL_FUNC (dialogCB), GINT_TO_POINTER (GX_CANCEL));
       GTK_WIDGET_SET_FLAGS (w, GTK_CAN_DEFAULT);
       gtk_widget_grab_default (w);
       gtk_widget_show (w);

    }
    
    gtk_window_set_position (GTK_WINDOW (window), GTK_WIN_POS_MOUSE);
    gtk_widget_show (window);
    gtk_grab_add (window);
    
    while (state->get_another)
        gtk_main_iteration ();
	            
    gtk_grab_remove (window);
    gdk_pixmap_unref (gdk_pixmap);    
    gtk_widget_destroy (window);
    
    retval = GPOINTER_TO_INT (state->retval);    
    g_free (state);
    
    
    return (retval);

} /* END */

 

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
gchar *gxinput (const gchar *message_str, const gchar *init_str,
    const gchar *utitle)
{
    GtkWidget *window;
    GtkWidget *vBox, *hBox, *w, *entry;

    gchar **discriminator;
    gchar *title;
    static gchar retval[256];


    /*
    ** Global state data
    */ 
    state = g_malloc (sizeof (*state));

    /* 
    ** discriminator holds the array of char pointers obtained from the
    ** #include of the pixmaps
    */
    discriminator = info_xpm;
    title =  utitle ? (gchar *) utitle : "Input";
       
    /*
    ** Event processing flag: process events from dialog while TRUE
    */       
    state->get_another = TRUE;

    /*
    ** Toplevel.
    */
    window = gtk_window_new (GTK_WINDOW_TOPLEVEL);    
    gtk_signal_connect (GTK_OBJECT (window), "destroy",
	GTK_SIGNAL_FUNC (gtk_widget_destroy), NULL);
    gtk_window_set_title (GTK_WINDOW (window), title);
    gtk_container_set_border_width (GTK_CONTAINER (window), 10);
    
    vBox = gtk_vbox_new (FALSE, 10);
    gtk_container_add (GTK_CONTAINER (window), vBox);
    gtk_widget_show (vBox);    
    
    w = gtk_label_new (message_str);
    gtk_box_pack_start (GTK_BOX (vBox), w, FALSE, FALSE, 2);
    gtk_label_set_justify (GTK_LABEL (w), GTK_JUSTIFY_LEFT);
    gtk_widget_show (w);

    entry = gtk_entry_new ();
    if (init_str) {
       gtk_entry_set_text (GTK_ENTRY (entry), init_str);
       gtk_entry_select_region (GTK_ENTRY (entry), 0, strlen (init_str));
    }
    gtk_box_pack_start (GTK_BOX (vBox), entry, FALSE, FALSE, 2);
    gtk_signal_connect (GTK_OBJECT (entry), "activate",
	GTK_SIGNAL_FUNC (inputCB), GINT_TO_POINTER (GX_OK));
    gtk_object_set_user_data (GTK_OBJECT (entry), entry);
    gtk_window_set_focus (GTK_WINDOW (window), entry);
    gtk_widget_show (entry);

    /* - - - - - - - - - - */

    w = gtk_hseparator_new ();
    gtk_box_pack_start (GTK_BOX (vBox), w, FALSE, FALSE, 2);
    gtk_widget_show (w);

    hBox = gtk_hbox_new (TRUE, 10);
    gtk_box_pack_start (GTK_BOX (vBox), hBox, FALSE, FALSE, 2);
    gtk_widget_show (hBox);

    w = gtk_button_new_with_label ("Ok");
    gtk_box_pack_start (GTK_BOX (hBox), w, TRUE, TRUE, 0);
    gtk_signal_connect (GTK_OBJECT (w), "clicked",
	GTK_SIGNAL_FUNC (inputCB), GINT_TO_POINTER (GX_OK));
    GTK_WIDGET_SET_FLAGS (w, GTK_CAN_DEFAULT);
    gtk_widget_grab_default (w);
    gtk_object_set_user_data (GTK_OBJECT (w), entry);
    gtk_widget_show (w);

    w = gtk_button_new_with_label ("Cancel");
    gtk_box_pack_start (GTK_BOX (hBox), w, TRUE, TRUE, 0);
    gtk_signal_connect (GTK_OBJECT (w), "clicked",
	GTK_SIGNAL_FUNC (inputCB), GINT_TO_POINTER (GX_CANCEL));
    gtk_object_set_user_data (GTK_OBJECT (w), entry);
    gtk_widget_show (w);
    
    gtk_window_set_position (GTK_WINDOW (window), GTK_WIN_POS_MOUSE);
    gtk_widget_show (window);
    gtk_grab_add (window);
    
    while (state->get_another)
        gtk_main_iteration ();
	                
    strcpy (retval, state->retval);
        
    gtk_grab_remove (window);
    gtk_widget_destroy (window);

    g_free (state);

    
    if (strcmp (retval, "") == 0)
       return (NULL);
           
    return (retval);

} /* END */



/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
void usage(void)
{
   g_warning ("gxdialog: invalid function call:\n");
   g_print ("gint gxdialog (const gchar *type_str, const gchar *message_str)\n");
   g_print ("    type_str = \"error\"\n");
   g_print ("               \"info\"\n");
   g_print ("               \"question\"\n");
   g_print ("               \"warning\"\n");
   g_print ("               \"about\"\n");
   g_print ("               \"help\"\n");
   g_print ("               \"gnome\"\n");
   g_print ("               \"kde\"\n");
   g_print ("    message_string = the message drawn in the dialog box\n");

}




/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
**
**  LIBRARY
**
** * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

/* Dont want a main() in the library! */

#undef DO_GXDIALOG_DEMO
#ifdef DO_GXDIALOG_DEMO

gint main (gint argc, gchar **argv)
{
    GtkWidget *window;
    GtkWidget *vBox, *w;


    gtk_init (&argc, &argv);
        
    window = gtk_window_new (GTK_WINDOW_TOPLEVEL);    
    gtk_signal_connect (GTK_OBJECT (window), "destroy",
	GTK_SIGNAL_FUNC (gtk_widget_destroy), NULL);
    gtk_window_set_title (GTK_WINDOW (window), "gxdialog");
    gtk_container_set_border_width (GTK_CONTAINER (window), 10);
    gtk_widget_set_usize (window, 200, 90);
	 
    vBox = gtk_vbox_new (FALSE, 10);
    gtk_container_add (GTK_CONTAINER (window), vBox);
    gtk_widget_show (vBox);    

    w = gtk_button_new_with_label ("Test gxdialog");
    gtk_box_pack_start (GTK_BOX (vBox), w, TRUE, TRUE, 0);
    gtk_signal_connect (GTK_OBJECT (w), "clicked",
	GTK_SIGNAL_FUNC (demo), NULL);
    GTK_WIDGET_SET_FLAGS (w, GTK_CAN_DEFAULT);
    gtk_widget_grab_default (w);
    gtk_widget_show (w);

    w = gtk_button_new_with_label ("Dismiss");
    gtk_box_pack_start (GTK_BOX (vBox), w, TRUE, TRUE, 0);
    gtk_signal_connect_object (GTK_OBJECT (w), "clicked",
	GTK_SIGNAL_FUNC (gtk_exit), GTK_OBJECT (window));
    gtk_widget_show (w);


    gtk_widget_show (window);

    gtk_main ();


    return (1);
    
}

void demo (GtkWidget *widget, gpointer data)
{
    gint x;
    gchar *answer;    
    gchar *text;

    const gchar *author = "Robert S. Mallozzi";
    const gchar *email  = "mallozzir@cspar.uah.edu";
    const gchar *url    = "http://cspar.uah.edu/~mallozzir/";

    
    
    x = gxdialog ("info", "Some Info", NULL);
    g_print ("info dialog:\t\treturn value = %d\n", x);

    x = gxdialog ("question", "File foo.c exists.\nOk to overwrite?", NULL);
    g_print ("question dialog:\treturn value = %d\n", x);

    x = gxdialog ("warning", "Warning: Invalid Filename!", "Oops");
    g_print ("warning dialog:\t\treturn value = %d\n", x);

    x = gxdialog ("error", "An Error Occurred.", "Bummer");
    g_print ("error dialog:\t\treturn value = %d\n", x);

    x = gxdialog ("info",
        "Here is an example of a long label.\n"
        "Here is an example of a long label.\n"
        "Here is an example of a long label.\n"
        "Here is an example of a long label.\n"
        "Here is an example of a long label.\n"
	"Wrapping is not automatic - you have\n"
	"to insert line breaks explicitly.", 
	NULL);
    g_print ("info dialog:\t\treturn value = %d\n", x);
       
    answer = gxinput ("Enter some text", "Initial text", NULL);
    g_print ("You entered %s\n", answer);
     
    answer = gxinput ("Enter some more text.\nNow there is no initial input", 
        NULL, NULL);
    g_print ("You entered %s\n", answer);


    x = gxdialog ("help", "Good for short help dialogs.", NULL);
    g_print ("help dialog:\t\treturn value = %d\n", x);

    x = gxdialog ("gnome", "Another cool dialog!", NULL);
    g_print ("gnome dialog:\t\treturn value = %d\n", x);

    x = gxdialog ("kde", "And another!", NULL);
    g_print ("kde dialog:\t\treturn value = %d\n", x);

    text = (gchar *) g_malloc (
	strlen ("gxdialog") + 2 + 
	strlen (author) + 1 +
	strlen (email) + 1 +
	strlen (url) + 1
    );

    sprintf (text, "gxdialog\n\n%s\n%s\n%s\n", author, email, url);	 
    
    x = gxdialog ("about", text, "The End");
    g_print ("about dialog:\t\treturn value = %d\n", x);

    g_free (text);


}

#endif /* GXDIALOG_DEMO */
